@@ -5,13 +5,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! evalexpr = "5"
-//! ```
-//!
-//! Add the `extern crate` definition to your `main.rs` or `lib.rs`:
-//!
-//! ```rust
-//! extern crate evalexpr;
+//! evalexpr = "6"
 //! ```
 //!
 //! Then you can use `evalexpr` to **evaluate expressions** like this:
@@ -56,7 +50,7 @@
 //! let context = context_map! {
 //!     "five" => 5,
 //!     "twelve" => 12,
-//!     "f" => Function::new(Box::new(|argument| {
+//!     "f" => Function::new(|argument| {
 //!         if let Ok(int) = argument.as_int() {
 //!             Ok(Value::Int(int / 2))
 //!         } else if let Ok(float) = argument.as_float() {
@@ -64,8 +58,8 @@
 //!         } else {
 //!             Err(EvalexprError::expected_number(argument.clone()))
 //!         }
-//!     })),
-//!     "avg" => Function::new(Box::new(|argument| {
+//!     }),
+//!     "avg" => Function::new(|argument| {
 //!         let arguments = argument.as_tuple()?;
 //!
 //!         if let (Value::Int(a), Value::Int(b)) = (&arguments[0], &arguments[1]) {
@@ -73,7 +67,7 @@
 //!         } else {
 //!             Ok(Value::Float((arguments[0].as_number()? + arguments[1].as_number()?) / 2.0))
 //!         }
-//!     }))
+//!     })
 //! }.unwrap(); // Do proper error handling here
 //!
 //! assert_eq!(eval_with_context("five + 8 > f(twelve)", &context), Ok(Value::from(true)));
@@ -196,7 +190,7 @@
 //! use evalexpr::*;
 //!
 //! let mut context = HashMapContext::new();
-//! assert_eq!(eval_with_context("a = 5", &context), Err(EvalexprError::ContextNotManipulable));
+//! assert_eq!(eval_with_context("a = 5", &context), Err(EvalexprError::ContextNotMutable));
 //! assert_eq!(eval_empty_with_context_mut("a = 5", &mut context), Ok(EMPTY_VALUE));
 //! assert_eq!(eval_empty_with_context_mut("a = 5.0", &mut context),
 //!            Err(EvalexprError::expected_int(5.0.into())));
@@ -259,7 +253,7 @@
 //! let mut context = HashMapContext::new();
 //! assert_eq!(eval_with_context_mut("a = 5;", &mut context), Ok(Value::from(())));
 //! // Assignments require mutable contexts
-//! assert_eq!(eval_with_context("a = 6", &context), Err(EvalexprError::ContextNotManipulable));
+//! assert_eq!(eval_with_context("a = 6", &context), Err(EvalexprError::ContextNotMutable));
 //! // The HashMapContext is type safe
 //! assert_eq!(eval_with_context_mut("a = 5.5", &mut context),
 //!            Err(EvalexprError::ExpectedInt { actual: Value::from(5.5) }));
@@ -302,7 +296,7 @@
 //! use evalexpr::*;
 //!
 //! let context = context_map!{
-//!     "f" => Function::new(Box::new(|args| Ok(Value::from(args.as_int()? + 5)))),
+//!     "f" => Function::new(|args| Ok(Value::from(args.as_int()? + 5))),
 //! }.unwrap_or_else(|error| panic!("Error creating context: {}", error));
 //! assert_eq!(eval_int_with_context("f 5", &context), Ok(10));
 //! ```
@@ -475,6 +469,7 @@
 //!
 
 #![deny(missing_docs)]
+#![forbid(unsafe_code)]
 
 #[cfg(feature = "regex_support")]
 extern crate regex;
@@ -487,7 +482,10 @@ extern crate serde;
 extern crate serde_derive;
 
 pub use crate::{
-    context::{Context, EmptyContext, HashMapContext},
+    context::{
+        Context, ContextWithMutableFunctions, ContextWithMutableVariables, EmptyContext,
+        HashMapContext,
+    },
     error::{EvalexprError, EvalexprResult},
     function::Function,
     interface::*,
